Method: EY::Repo#initialize

Defined in:
lib/engineyard/repo.rb

#initializeRepo

$GIT_DIR is what git uses to override the location of the .git dir. $GIT_WORK_TREE is the working tree for git, which we'll use after $GIT_DIR.

We use this to specify which repo we should look at, since it would also specify where any git commands are directed, thus fooling commands we run anyway.



33
34
# File 'lib/engineyard/repo.rb', line 33

def initialize
end