Module: Thor::Base::ClassMethods
- Defined in:
- lib/vendor/thor/lib/thor/base.rb
Instance Method Summary collapse
-
#all_tasks ⇒ Object
Returns the tasks for this Thor class and all subclasses.
-
#argument(name, options = {}) ⇒ Object
Adds an argument to the class and creates an attr_accessor for it.
-
#arguments ⇒ Object
Returns this class arguments, looking up in the ancestors chain.
-
#attr_accessor ⇒ Object
:nodoc:.
-
#attr_reader ⇒ Object
:nodoc:.
-
#attr_writer ⇒ Object
:nodoc:.
-
#check_unknown_options ⇒ Object
:nodoc:.
-
#check_unknown_options! ⇒ Object
If you want to raise an error for unknown options, call check_unknown_options! This is disabled by default to allow dynamic invocations.
-
#check_unknown_options?(config) ⇒ Boolean
:nodoc:.
-
#class_option(name, options = {}) ⇒ Object
Adds an option to the set of class options.
-
#class_options(options = nil) ⇒ Object
Adds a bunch of options to the set of class options.
-
#group(name = nil) ⇒ Object
Defines the group.
-
#handle_argument_error(task, error, arity = nil) ⇒ Object
:nodoc:.
-
#handle_no_task_error(task, has_namespace = $thor_runner) ⇒ Object
:nodoc:.
-
#namespace(name = nil) ⇒ Object
Sets the namespace for the Thor or Thor::Group class.
-
#no_tasks ⇒ Object
All methods defined inside the given block are not added as tasks.
-
#public_task(*names) ⇒ Object
Allows to use private methods from parent in child classes as tasks.
-
#remove_argument(*names) ⇒ Object
Removes a previous defined argument.
-
#remove_class_option(*names) ⇒ Object
Removes a previous defined class option.
-
#remove_task(*names) ⇒ Object
Removes a given task from this Thor class.
-
#start(given_args = ARGV, config = {}) ⇒ Object
Parses the task and options from the given args, instantiate the class and invoke the task.
-
#tasks ⇒ Object
Returns the tasks for this Thor class.
Instance Method Details
#all_tasks ⇒ Object
Returns the tasks for this Thor class and all subclasses.
Returns
- OrderedHash
An ordered hash with tasks names as keys and Thor::Task objects as values.
314 315 316 317 |
# File 'lib/vendor/thor/lib/thor/base.rb', line 314 def all_tasks @all_tasks ||= from_superclass(:all_tasks, Thor::CoreExt::OrderedHash.new) @all_tasks.merge(tasks) end |
#argument(name, options = {}) ⇒ Object
Adds an argument to the class and creates an attr_accessor for it.
Arguments are different from options in several aspects. The first one is how they are parsed from the command line, arguments are retrieved from position:
thor task NAME
Instead of:
thor task --name=NAME
Besides, arguments are used inside your code as an accessor (self.argument), while options are all kept in a hash (self.options).
Finally, arguments cannot have type :default or :boolean but can be optional (supplying :optional => :true or :required => false), although you cannot have a required argument after a non-required argument. If you try it, an error is raised.
Parameters
- name
The name of the argument.
- options
Described below.
Options
:desc - Description for the argument. :required - If the argument is required or not. :optional - If the argument is optional or not. :type - The type of the argument, can be :string, :hash, :array, :numeric. :default - Default value for this argument. It cannot be required and have default values. :banner - String to show on usage notes.
Errors
- ArgumentError
Raised if you supply a required argument after a non required one.
179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 |
# File 'lib/vendor/thor/lib/thor/base.rb', line 179 def argument(name, ={}) is_thor_reserved_word?(name, :argument) no_tasks { attr_accessor name } required = if .key?(:optional) ![:optional] elsif .key?(:required) [:required] else [:default].nil? end remove_argument name arguments.each do |argument| next if argument.required? raise ArgumentError, "You cannot have #{name.to_s.inspect} as required argument after " << "the non-required argument #{argument.human_name.inspect}." end if required arguments << Thor::Argument.new(name, [:desc], required, [:type], [:default], [:banner]) end |
#arguments ⇒ Object
208 209 210 |
# File 'lib/vendor/thor/lib/thor/base.rb', line 208 def arguments @arguments ||= from_superclass(:arguments, []) end |
#attr_accessor ⇒ Object
:nodoc:
126 127 128 |
# File 'lib/vendor/thor/lib/thor/base.rb', line 126 def attr_accessor(*) #:nodoc: no_tasks { super } end |
#attr_reader ⇒ Object
:nodoc:
118 119 120 |
# File 'lib/vendor/thor/lib/thor/base.rb', line 118 def attr_reader(*) #:nodoc: no_tasks { super } end |
#attr_writer ⇒ Object
:nodoc:
122 123 124 |
# File 'lib/vendor/thor/lib/thor/base.rb', line 122 def attr_writer(*) #:nodoc: no_tasks { super } end |
#check_unknown_options ⇒ Object
:nodoc:
136 137 138 |
# File 'lib/vendor/thor/lib/thor/base.rb', line 136 def #:nodoc: @check_unknown_options ||= from_superclass(:check_unknown_options, false) end |
#check_unknown_options! ⇒ Object
If you want to raise an error for unknown options, call check_unknown_options! This is disabled by default to allow dynamic invocations.
132 133 134 |
# File 'lib/vendor/thor/lib/thor/base.rb', line 132 def @check_unknown_options = true end |
#check_unknown_options?(config) ⇒ Boolean
:nodoc:
140 141 142 |
# File 'lib/vendor/thor/lib/thor/base.rb', line 140 def (config) #:nodoc: !! end |
#class_option(name, options = {}) ⇒ Object
Adds an option to the set of class options
Parameters
- name
The name of the argument.
- options
Described below.
Options
:desc:: -- Description for the argument. :required:: -- If the argument is required or not. :default:: -- Default value for this argument. :group:: -- The group for this options. Use by class options to output options in different levels. :aliases:: -- Aliases for this option. Note: Thor follows a convention of one-dash-one-letter options. Thus aliases like "-something" wouldn't be parsed; use either "--something" or "-s" instead. :type:: -- The type of the argument, can be :string, :hash, :array, :numeric or :boolean. :banner:: -- String to show on usage notes. :hide:: -- If you want to hide this option from the help.
243 244 245 |
# File 'lib/vendor/thor/lib/thor/base.rb', line 243 def class_option(name, ={}) build_option(name, , ) end |
#class_options(options = nil) ⇒ Object
Adds a bunch of options to the set of class options.
:foo => false, :bar => :required, :baz => :string
If you prefer more detailed declaration, check class_option.
Parameters
Hash[Symbol => Object]
221 222 223 224 225 |
# File 'lib/vendor/thor/lib/thor/base.rb', line 221 def (=nil) @class_options ||= from_superclass(:class_options, {}) (, @class_options) if @class_options end |
#group(name = nil) ⇒ Object
Defines the group. This is used when thor list is invoked so you can specify that only tasks from a pre-defined group will be shown. Defaults to standard.
Parameters
name<String|Symbol>
289 290 291 292 293 294 295 296 |
# File 'lib/vendor/thor/lib/thor/base.rb', line 289 def group(name=nil) case name when nil @group ||= from_superclass(:group, 'standard') else @group = name.to_s end end |
#handle_argument_error(task, error, arity = nil) ⇒ Object
:nodoc:
438 439 440 441 442 443 444 445 446 447 448 449 450 |
# File 'lib/vendor/thor/lib/thor/base.rb', line 438 def handle_argument_error(task, error, arity=nil) #:nodoc: msg = "#{basename} #{task.name}" if arity required = arity < 0 ? (-1 - arity) : arity msg << " requires at least #{required} argument" msg << "s" if required > 1 else msg = "call #{msg} as" end msg << ": #{self.(task).inspect}." raise InvocationError, msg end |
#handle_no_task_error(task, has_namespace = $thor_runner) ⇒ Object
:nodoc:
430 431 432 433 434 435 436 |
# File 'lib/vendor/thor/lib/thor/base.rb', line 430 def handle_no_task_error(task, has_namespace = $thor_runner) #:nodoc: if has_namespace raise UndefinedTaskError, "Could not find task #{task.inspect} in #{namespace.inspect} namespace." else raise UndefinedTaskError, "Could not find task #{task.inspect}." end end |
#namespace(name = nil) ⇒ Object
Sets the namespace for the Thor or Thor::Group class. By default the namespace is retrieved from the class name. If your Thor class is named Scripts::MyScript, the help method, for example, will be called as:
thor scripts:my_script -h
If you change the namespace:
namespace :my_scripts
You change how your tasks are invoked:
thor my_scripts -h
Finally, if you change your namespace to default:
namespace :default
Your tasks can be invoked with a shortcut. Instead of:
thor :my_task
389 390 391 392 393 394 395 396 |
# File 'lib/vendor/thor/lib/thor/base.rb', line 389 def namespace(name=nil) case name when nil @namespace ||= Thor::Util.namespace_from_thor_class(self) else @namespace = name.to_s end end |
#no_tasks ⇒ Object
360 361 362 363 364 365 |
# File 'lib/vendor/thor/lib/thor/base.rb', line 360 def no_tasks @no_tasks = true yield ensure @no_tasks = false end |
#public_task(*names) ⇒ Object
Allows to use private methods from parent in child classes as tasks.
Parameters
- names
Method names to be used as tasks
Examples
public_task :foo
public_task :foo, :bar, :baz
424 425 426 427 428 |
# File 'lib/vendor/thor/lib/thor/base.rb', line 424 def public_task(*names) names.each do |name| class_eval "def #{name}(*); super end" end end |
#remove_argument(*names) ⇒ Object
Removes a previous defined argument. If :undefine is given, undefine accessors as well.
Parameters
- names
Arguments to be removed
Examples
remove_argument :foo
remove_argument :foo, :bar, :baz, :undefine => true
258 259 260 261 262 263 264 265 |
# File 'lib/vendor/thor/lib/thor/base.rb', line 258 def remove_argument(*names) = names.last.is_a?(Hash) ? names.pop : {} names.each do |name| arguments.delete_if { |a| a.name == name.to_s } undef_method name, "#{name}=" if [:undefine] end end |
#remove_class_option(*names) ⇒ Object
Removes a previous defined class option.
Parameters
- names
Class options to be removed
Examples
remove_class_option :foo
remove_class_option :foo, :bar, :baz
277 278 279 280 281 |
# File 'lib/vendor/thor/lib/thor/base.rb', line 277 def remove_class_option(*names) names.each do |name| .delete(name) end end |
#remove_task(*names) ⇒ Object
Removes a given task from this Thor class. This is usually done if you are inheriting from another class and don't want it to be available anymore.
By default it only remove the mapping to the task. But you can supply :undefine => true to undefine the method from the class as well.
Parameters
- name<Symbol|String>
The name of the task to be removed
- options
You can give :undefine => true if you want tasks the method to be undefined from the class as well.
331 332 333 334 335 336 337 338 339 |
# File 'lib/vendor/thor/lib/thor/base.rb', line 331 def remove_task(*names) = names.last.is_a?(Hash) ? names.pop : {} names.each do |name| tasks.delete(name.to_s) all_tasks.delete(name.to_s) undef_method name if [:undefine] end end |
#start(given_args = ARGV, config = {}) ⇒ Object
Parses the task and options from the given args, instantiate the class and invoke the task. This method is used when the arguments must be parsed from an array. If you are inside Ruby and want to use a Thor class, you can simply initialize it:
script = MyScript.new(args, , config)
script.invoke(:task, first_arg, second_arg, third_arg)
406 407 408 409 410 411 412 |
# File 'lib/vendor/thor/lib/thor/base.rb', line 406 def start(given_args=ARGV, config={}) config[:shell] ||= Thor::Base.shell.new dispatch(nil, given_args.dup, nil, config) rescue Thor::Error => e ENV["THOR_DEBUG"] == "1" ? (raise e) : config[:shell].error(e.) exit(1) if exit_on_failure? end |
#tasks ⇒ Object
Returns the tasks for this Thor class.
Returns
- OrderedHash
An ordered hash with tasks names as keys and Thor::Task objects as values.
304 305 306 |
# File 'lib/vendor/thor/lib/thor/base.rb', line 304 def tasks @tasks ||= Thor::CoreExt::OrderedHash.new end |